#ifdef GL_ES
precision mediump float;
#endif

// 1
varying vec2 v_texCoord;
uniform sampler2D u_texture;
uniform vec3 v_hsv; // offset

float
minChannel(in vec3 v)
{
  float t = (v.x<v.y) ? v.x : v.y;
  t = (t<v.z) ? t : v.z;
  return t;
}

float
maxChannel(in vec3 v)
{
  float t = (v.x>v.y) ? v.x : v.y;
  t = (t>v.z) ? t : v.z;
  return t;
}

vec3
rgbToHsv(in vec3 rgb)
{
  vec3 hsv = vec3(0.0);
  float minVal = minChannel(rgb);
  float maxVal = maxChannel(rgb);
  float delta = maxVal - minVal;

  hsv.z = maxVal;

  if (delta != 0.0) {
    hsv.y = delta / maxVal;
    vec3 delRGB;
    delRGB = (((vec3(maxVal) - rgb) / 6.0) + (delta/2.0)) / delta;

    if (rgb.x == maxVal) {
      hsv.x = delRGB.z - delRGB.y;
    } else if (rgb.y == maxVal) {
      hsv.x = ( 1.0/3.0) + delRGB.x - delRGB.z;
    } else if (rgb.z == maxVal) {
      hsv.x = ( 2.0/3.0) + delRGB.y - delRGB.x;
    }

    if ( hsv.x < 0.0 ) {
      hsv.x += 1.0;
    }
    if ( hsv.x > 1.0 ) {
      hsv.x -= 1.0;
    }
  }
  return hsv;
}

vec3
hsvToRgb(in vec3 hsv)
{
  int i;
  vec3 rgb = vec3(hsv.z);
  if ( hsv.y != 0.0 ) {
    float var_h = hsv.x * 6.0;
    float var_i = floor(var_h); // Or ... var_i = floor( var_h )
    float var_1 = hsv.z * (1.0 - hsv.y);
    float var_2 = hsv.z * (1.0 - hsv.y * (var_h-var_i));
    float var_3 = hsv.z * (1.0 - hsv.y * (1.0 - (var_h-var_i)));

	i = int(var_i);
    if (i == 0) rgb = vec3(hsv.z, var_3, var_1);
	else if (i == 1) rgb = vec3(var_2, hsv.z, var_1);
    else if (i == 2) rgb = vec3(var_1, hsv.z, var_3);
    else if (i == 3) rgb = vec3(var_1, var_2, hsv.z);
    else if (i == 4) rgb = vec3(var_3, var_1, hsv.z);
    else rgb = vec3(hsv.z, var_1, var_2);
  }
  return rgb;
}

vec3 rgb2hsv(vec3 c)
{
    vec4 K = vec4(0.0, -1.0 / 3.0, 2.0 / 3.0, -1.0);
    vec4 p = mix(vec4(c.bg, K.wz), vec4(c.gb, K.xy), step(c.b, c.g));
    vec4 q = mix(vec4(p.xyw, c.r), vec4(c.r, p.yzx), step(p.x, c.r));

    float d = q.x - min(q.w, q.y);
    float e = 1.0e-10;
    return vec3(abs(q.z + (q.w - q.y) / (6.0 * d + e)), d / (q.x + e), q.x);
}

vec3 hsv2rgb(vec3 c)
{
    vec4 K = vec4(1.0, 2.0 / 3.0, 1.0 / 3.0, 3.0);
    vec3 p = abs(fract(c.xxx + K.xyz) * 6.0 - K.www);
    return c.z * mix(K.xxx, clamp(p - K.xxx, 0.0, 1.0), c.y);
}


void main()
{ // 2
    vec4 normalColor = texture2D(u_texture, v_texCoord).rgba;

	vec3 color;
	color.r = normalColor.r;
	color.g = normalColor.g;
	color.b = normalColor.b;

	color = rgbToHsv(color);

	color += v_hsv;
    color.x = mod(color.x, 1.0);
    if (color.y > 1.0)
		color.y = 1.0;
	else if (color.y < 0.0)
		color.y = 0.0;
    if (color.z > 1.0)
		color.z = 1.0;
	else if (color.z < 0.0)
		color.z = 0.0;

	color = hsvToRgb(color);
	
    // 4
	gl_FragColor = vec4(color.r, color.g, color.b, normalColor.a);

}